package com.ozs.web.controller.pm;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ozs.common.core.domain.model.LoginUser;
import com.ozs.framework.web.service.TokenService;
import com.ozs.pm.doman.vo.requestVo.PmBookBuildingReqVo;
import com.ozs.pm.doman.vo.requestVo.PmDemandReqVo;
import com.ozs.pm.doman.vo.responseVo.PmDemandResVo;
import com.ozs.pm.service.IPmDemandService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;

import javax.servlet.http.HttpServletRequest;

/**
 * 采购需求Controller
 *
 * @author ruoyi
 * @date 2023-01-16
 */
@Api(tags = "采购需求")
@RestController
@RequestMapping("/pm/demand")
public class PmDemandController extends BaseController {
    @Autowired
    private IPmDemandService pmDemandService;
    @Autowired
    private TokenService tokenService;

    /**
     * 查询采购需求列表
     */
    @ApiOperation(value = "查询采购需求列表", notes = "参数非必传")
    @PostMapping("/list")
    public AjaxResult list(@RequestBody PmDemandReqVo pmDemandReqVo) {
        IPage<PmDemandResVo> page  = pmDemandService.selectPmDemandList(pmDemandReqVo,0);
        return success(page);

    }

    /**
     * 获取采购需求详细信息
     */
    @ApiOperation(value = "获取采购需求详细信息", notes = "必传demandId,其他字段不传")
    @PostMapping("/view")
    public AjaxResult view(@RequestBody PmDemandReqVo pmDemandReqVo) {
        return success(pmDemandService.selectPmDemandByDemandId(pmDemandReqVo.getDemandId()));
    }

    /**
     * 需求建档
     */
    @ApiOperation(value = "需求建档", notes = "必传demandId,根据项目类型必传pmDemandEngineeringResponseVo、pmDemandEquipResponseVo、pmDemandMaterialsResponseVo、pmDemandServeResponseVo")
    @PostMapping("/bookBuilding")
    public AjaxResult bookBuilding(@RequestBody PmBookBuildingReqVo pmBookBuildingReqVo, HttpServletRequest request) {
        LoginUser loginUser = tokenService.getLoginUser(request);
        pmBookBuildingReqVo.setUpdateBy(String.valueOf(loginUser.getUserId()));
        return toAjax(pmDemandService.bookBuilding(pmBookBuildingReqVo));
    }

    @ApiOperation(value = "提交采购需求", notes = "必传demandId,其他字段不传")
    @PostMapping("/commit")
    public AjaxResult commit(@RequestBody PmDemandReqVo pmDemandReqVo, HttpServletRequest request) {
        LoginUser loginUser = tokenService.getLoginUser(request);
        pmDemandReqVo.setUpdateBy(String.valueOf(loginUser.getUserId()));
        return toAjax(pmDemandService.commit(pmDemandReqVo));
    }

    /**
     * 查看流程图，获取当前阶段名字
     */
    @ApiOperation(value = "查看流程图，获取当前阶段名字", notes = "必传demandId,其他字段不传")
    @PostMapping("/viewFlowChart")
    public AjaxResult viewFlowChart(@RequestBody PmDemandReqVo pmRequestVo) {
        return success(pmDemandService.viewFlowChart(pmRequestVo.getDemandId()));
    }

    /**
     * 查看流程图，鼠标移到对应模块后，显示悬浮框提示
     */
    @ApiOperation(value = "查看流程图，鼠标移到对应模块后，显示悬浮框提示", notes = "必传demandId,模块名称，其他字段不传")
    @PostMapping("/getModuleInfo")
    public AjaxResult getModuleInfo(@RequestBody PmDemandReqVo pmRequestVo) {
        return success(pmDemandService.getModuleInfo(pmRequestVo.getDemandId(),pmRequestVo.getModuleName()));
    }


    /**
     * 审核单位查询采购需求列表
     */
    @ApiOperation(value = "审核单位查询采购需求列表", notes = "参数非必传")
    @PostMapping("/examineList")
    public AjaxResult examineList(@RequestBody PmDemandReqVo pmDemandReqVo) {
        IPage<PmDemandResVo> page  = pmDemandService.selectPmDemandList(pmDemandReqVo,1);
        return success(page);
    }

    @ApiOperation(value = "审核采购需求通过", notes = "必传demandId和上传附件，其他字段不传")
    @PostMapping("/reviewTo")
    public AjaxResult reviewTo(@RequestBody PmDemandReqVo pmDemandReqVo, HttpServletRequest request) {
        LoginUser loginUser = tokenService.getLoginUser(request);
        pmDemandReqVo.setUpdateBy(String.valueOf(loginUser.getUserId()));
        return toAjax(pmDemandService.reviewTo(pmDemandReqVo));
    }

    @ApiOperation(value = "审核采购需求退回", notes = "必传demandId和退回原因，其他字段不传")
    @PostMapping("/reviewReturn")
    public AjaxResult reviewReturn(@RequestBody PmDemandReqVo pmDemandReqVo, HttpServletRequest request) {
        LoginUser loginUser = tokenService.getLoginUser(request);
        pmDemandReqVo.setUpdateBy(String.valueOf(loginUser.getUserId()));
        return toAjax(pmDemandService.reviewReturn(pmDemandReqVo));
    }

}
