package com.ozs.base.vo;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.ozs.common.annotation.Excel;
import com.ozs.common.core.domain.BaseEntity;
import com.ozs.system.domain.SysFileInfo;
import io.swagger.annotations.ApiModel;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;


@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
@ApiModel("项目信息")
public class BaseProjectInfo
{
    private static final long serialVersionUID = 1L;

    /*实际完成采购时间是指采购执行管理中，点击填制中标信息，弹出框中，填写的中标时间
    * 实际提报时间就是指采购需求管理中"提交"的时间*/

    /** 主键 */
    @TableId(type = IdType.AUTO)
    private Long demandId;

    /** 采购单位（登录账号的单位） */
    @Excel(name = "采购单位", readConverterExp = "登=录账号的单位")
    @TableField(exist = false)
    private String purchaseDeptName;

    /** 项目名称 */
    @Excel(name = "项目名称")
    private String projectName;

    /** 项目类型（0：装备类；1：物资类；2：服务类；3：工程类） */
    @Excel(name = "项目类型", readConverterExp = "0=：装备类；1：物资类；2：服务类；3：工程类")
    private String projectTypeStr;


    /** 概算金额(万元（保留小数点后两位） */
    @Excel(name = "概算金额(万元", readConverterExp = "保=留小数点后两位")
    private BigDecimal evaluation;


    /** 项目属性（0:应急应战项目， 1:重大规划任务项目， 2:规划任务项目， 3:一般项目） */
    @Excel(name = "项目属性", readConverterExp = "0=:应急应战项目，,1=:重大规划任务项目，,2=:规划任务项目，,3=:一般项目")
    private String projectAttrStr;


    /** 抽取招标代理机构id */
    @Excel(name = "抽取招标代理机构")
    private String agencyName;

    /** 抽取招标代理机构时间 */
    @Excel(name = "抽取招标代理机构时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date extractAgencyTime;



}
