package com.ozs.plan.service;


import com.github.pagehelper.PageInfo;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.model.LoginUser;
import com.ozs.plan.doman.PlanQuarter;
import com.ozs.plan.doman.vo.requestVo.PlanQuarterStandardVo;
import com.ozs.plan.doman.vo.requestVo.PlanYearsStandardVo;

import java.util.List;

/**
 * 季度计划 服务层
 *
 * @author bu
 */
public interface PlanQuarterService {

    /**
     * 查询季度计划记录集合
     *
     * @param vo 季度计划请求对象
     * @return 季度计划记录集合
     */
    public PageInfo<PlanQuarter> selectPlanQuarterList(PlanQuarterStandardVo vo);

    /**
     * 查询季度计划记录集合--导出
     *
     * @param vo 季度计划请求对象
     * @return 季度计划记录集合
     */
    public List<PlanQuarter> selectPlanQuarterListEXP(PlanQuarterStandardVo vo);

    /**
     * 查询季度计划记录集合(审核单位)
     *
     * @param vo 季度计划请求对象
     * @return 季度计划记录集合
     */
    public PageInfo<PlanQuarter> selectPlanQuarterExamineList(PlanQuarterStandardVo vo);

    /**
     * 查询季度计划记录集合(审核单位--导出）
     *
     * @param vo 季度计划请求对象
     * @return 季度计划记录集合
     */
    public List<PlanQuarter> selectPlanQuarterExamineListEXP(PlanQuarterStandardVo vo);

    /**
     * 创建季度计划
     *
     * @param quarterStandardVo 季度计划请求对象
     */
    public AjaxResult insertPlanQuarter(PlanQuarterStandardVo quarterStandardVo);

    /**
     * 批量删除季度计划
     *
     * @param planIds 需要删除的季度计划ID
     * @return 结果
     */
    public AjaxResult deletePlanQuarterByIds(Long[] planIds);

    /**
     * 批量删除季度计划
     *
     * @param planYearId 需要删除的季度计划ID
     * @return 结果
     */
    public AjaxResult deletePlanQuarterById(Long planYearId);

    /**
     * 根据id获取季度计划信息
     *
     * @param quarterStandardVo 季度计划请求对象
     * @return 结果
     */
    public AjaxResult view(PlanQuarterStandardVo quarterStandardVo);

    /**
     * 修改季度计划
     *
     * @param quarterStandardVo 修改季度计划请求对象
     * @return 结果
     */
    public AjaxResult update(PlanQuarterStandardVo quarterStandardVo);

    /**
     * 导入季度计划数据
     *
     * @return 结果
     */
    public String importPlanQuarter(List<PlanQuarterStandardVo> planquarter, boolean isUpdateSupport, LoginUser loginUser);

    /**
     * 提交季度计划
     *
     * @param quarterStandardVo 提交季度计划请求对象
     * @return 结果
     */
    public AjaxResult commit(PlanQuarterStandardVo quarterStandardVo);

    /**
     * 审核季度计划
     *
     * @param quarterStandardVo 需要审核的季度计划
     * @return 结果
     */
    public AjaxResult reviewTo(PlanQuarterStandardVo quarterStandardVo);

    /**
     * 审核季度计划退回
     *
     * @param quarterStandardVo 需要审核的季度计划
     * @return 结果
     */
    public AjaxResult reviewReturn(PlanQuarterStandardVo quarterStandardVo);

    /**
     * 申请修改季度计划
     *
     * @param quarterStandardVo 需要修改的季度计划
     * @return 结果
     */
    public AjaxResult appUpdate(PlanQuarterStandardVo quarterStandardVo);

    /**
     * 发函催告
     *
     * @param quarterStandardVo 季度计划
     * @return 结果
     */
    AjaxResult sendLetter(PlanQuarterStandardVo quarterStandardVo);

    /**
     * 上传计划关联文件后保存文件信息
     *
     * @param quarterStandardVo 季度计划
     * @return 结果
     */
    AjaxResult upLoadPlanFile(PlanQuarterStandardVo quarterStandardVo);

    /**
     * 下载计划关联文件
     *
     * @param quarterStandardVo 季度计划
     * @return 结果
     */
    AjaxResult downLoadPlanFile(PlanQuarterStandardVo quarterStandardVo);
}
